
DROP TABLE IF EXISTS `schema`;
DROP TABLE IF EXISTS `jobs`;
DROP TABLE IF EXISTS `job_types`;
DROP TABLE IF EXISTS `challenges`;
DROP TABLE IF EXISTS `moldable_job_descriptions`;
DROP TABLE IF EXISTS `job_resource_groups`;
DROP TABLE IF EXISTS `job_resource_descriptions`;
DROP TABLE IF EXISTS `job_state_logs`;
DROP TABLE IF EXISTS `frag_jobs`;
DROP TABLE IF EXISTS `assigned_resources`;
DROP TABLE IF EXISTS `resources`;
DROP TABLE IF EXISTS `resource_logs`;
DROP TABLE IF EXISTS `queues`;
DROP TABLE IF EXISTS `scheduler`;
DROP TABLE IF EXISTS `admission_rules`;
DROP TABLE IF EXISTS `gantt_jobs_predictions`;
DROP TABLE IF EXISTS `gantt_jobs_predictions_visu`;
DROP TABLE IF EXISTS `gantt_jobs_predictions_log`;
DROP TABLE IF EXISTS `gantt_jobs_resources`;
DROP TABLE IF EXISTS `gantt_jobs_resources_visu`;
DROP TABLE IF EXISTS `gantt_jobs_resources_log`;
DROP TABLE IF EXISTS `files`;
DROP TABLE IF EXISTS `event_logs`;
DROP TABLE IF EXISTS `event_log_hostnames`;
DROP TABLE IF EXISTS `accounting`;
DROP TABLE IF EXISTS `job_dependencies`;

