<!--
   BODY {
         background: white;
	 font-family: sans-serif;
	 }

h2 {
         color: #303030;
         font-family: sans-serif;
/*         border-bottom: thin solid; */
     }
			      
  h1,h3, h4, h5, h6 {
         color: #303030;
	 font-family: sans-serif;
/*         background-color: #C0C0C0;*/
/*	 border-bottom: thin solid; */
	 
   }
   pre {
	 border: 1px solid #CBCBCB;
	 background-color:  #ccccff;
	 color: black;
		}
table {
border-style:solid; 
border-width:1px; 
border-color:silver;
#width:50%;
}

td { 
border-style:solid; 
border-width:1px; 
border-color:white;
#width:50px;
} 

.fixed_td {
font-family: fixed;
}

.gray {
#border-style:solid; 
border-style:none; 
border-width:1px; 
border-color:white; 
background-color:silver;
}

.lgray {
#border-style:solid; 
border-style:none; 
border-width:1px; 
border-color:white; 
background-color:#cccccc;
border-collapse:collapse;
}

.lblue {
border-style:none; 
#border-width:1px; 
background-color:#9999ff;
border-collapse:collapse;
}
.bgray {
border-style:solid; 
border-width:1px; 
border-color:black; 
border-collapse:collapse
}
.none {
border-style:none;
}
-->
